@@ -5,11 +5,7 @@
 
 open Bigarray
 open Common
-
-type 'a reader = buf -> pos_ref:pos_ref -> 'a
-type ('a, 'b) reader1 = 'a reader -> 'b reader
-type ('a, 'b, 'c) reader2 = 'a reader -> ('b, 'c) reader1
-type ('a, 'b, 'c, 'd) reader3 = 'a reader -> ('b, 'c, 'd) reader2
+include Read_intf.Definitions
 
 external unsafe_get : buf -> int -> char = "%caml_ba_unsafe_ref_1"
 external unsafe_get8 : buf -> int -> int = "%caml_ba_unsafe_ref_1"
@@ -24,7 +20,7 @@ let unsafe_get8_signed buf pos =
 let arch_sixtyfour = Sys.word_size = 64
 let arch_big_endian = Sys.big_endian
 let max_int_int32 = if arch_sixtyfour then Int32.max_int else Int32.of_int max_int
-let min_int_int32 = if arch_sixtyfour then Int32.max_int else Int32.of_int min_int
+let min_int_int32 = if arch_sixtyfour then Int32.min_int else Int32.of_int min_int
 let max_int_int64 = Int64.of_int max_int
 let min_int_int64 = Int64.of_int min_int
 
@@ -47,10 +43,10 @@ let safe_nativeint_of_int64 =
   then fun _pos x -> Int64.to_nativeint x
   else
     fun [@inline always] pos x ->
-      if x >= Int64.of_nativeint Nativeint.min_int
-      && x <= Int64.of_nativeint Nativeint.max_int
-      then Int64.to_nativeint x
-      else raise_read_error ReadError.Int_overflow pos
+    if x >= Int64.of_nativeint Nativeint.min_int
+       && x <= Int64.of_nativeint Nativeint.max_int
+    then Int64.to_nativeint x
+    else raise_read_error ReadError.Int_overflow pos
 ;;
 
 external unsafe_get16 : buf -> int -> int = "%caml_bigstring_get16u"
@@ -210,15 +206,15 @@ let safe_bin_read_nat0_32 =
         Nat0.unsafe_of_int (n land mask_32bit))
   else
     fun buf ~pos_ref ~pos ->
-      let next = pos + 4 in
-      check_next buf next;
-      let n = unsafe_get32le buf pos in
-      if n >= 0l && n <= max_int_int32
-      then (
-        let n = Nat0.unsafe_of_int (Int32.to_int n) in
-        pos_ref := next;
-        n)
-      else raise_read_error ReadError.Nat0_overflow !pos_ref
+    let next = pos + 4 in
+    check_next buf next;
+    let n = unsafe_get32le buf pos in
+    if n >= 0l && n <= max_int_int32
+    then (
+      let n = Nat0.unsafe_of_int (Int32.to_int n) in
+      pos_ref := next;
+      n)
+    else raise_read_error ReadError.Nat0_overflow !pos_ref
 ;;
 
 let safe_bin_read_nat0_64 buf ~pos_ref ~pos =
