@@ -1,42 +1,6 @@
 (* Tp_class: sizers, writers, and readers in records *)
 
-type 'a writer =
-  { size : 'a Size.sizer
-  ; write : 'a Write.writer
-  }
-
-type 'a reader =
-  { read : 'a Read.reader
-  ; vtag_read : (int -> 'a) Read.reader
-  }
-
-type 'a t =
-  { shape : Shape.t
-  ; writer : 'a writer
-  ; reader : 'a reader
-  }
-
-type 'a writer0 = 'a writer
-type 'a reader0 = 'a reader
-type 'a t0 = 'a t
-
-module S1 = struct
-  type ('a, 'b) writer = 'a writer0 -> 'b writer0
-  type ('a, 'b) reader = 'a reader0 -> 'b reader0
-  type ('a, 'b) t = 'a t0 -> 'b t0
-end
-
-module S2 = struct
-  type ('a, 'b, 'c) writer = 'a writer0 -> ('b, 'c) S1.writer
-  type ('a, 'b, 'c) reader = 'a reader0 -> ('b, 'c) S1.reader
-  type ('a, 'b, 'c) t = 'a t0 -> ('b, 'c) S1.t
-end
-
-module S3 = struct
-  type ('a, 'b, 'c, 'd) writer = 'a writer0 -> ('b, 'c, 'd) S2.writer
-  type ('a, 'b, 'c, 'd) reader = 'a reader0 -> ('b, 'c, 'd) S2.reader
-  type ('a, 'b, 'c, 'd) t = 'a t0 -> ('b, 'c, 'd) S2.t
-end
+include Type_class_intf.Definitions
 
 let variant_wrong_type name _buf ~pos_ref _x =
   Common.raise_variant_wrong_type name !pos_ref
@@ -239,14 +203,14 @@ let bin_writer_pair bin_writer_el1 bin_writer_el2 =
   { size = (fun v -> Size.bin_size_pair bin_writer_el1.size bin_writer_el2.size v)
   ; write =
       (fun buf ~pos v ->
-         Write.bin_write_pair bin_writer_el1.write bin_writer_el2.write buf ~pos v)
+        Write.bin_write_pair bin_writer_el1.write bin_writer_el2.write buf ~pos v)
   }
 ;;
 
 let bin_reader_pair bin_reader_el1 bin_reader_el2 =
   { read =
       (fun buf ~pos_ref ->
-         Read.bin_read_pair bin_reader_el1.read bin_reader_el2.read buf ~pos_ref)
+        Read.bin_read_pair bin_reader_el1.read bin_reader_el2.read buf ~pos_ref)
   ; vtag_read = variant_wrong_type "pair"
   }
 ;;
@@ -264,28 +228,28 @@ let bin_pair bin_el1 bin_el2 =
 let bin_writer_triple bin_writer_el1 bin_writer_el2 bin_writer_el3 =
   { size =
       (fun v ->
-         Size.bin_size_triple bin_writer_el1.size bin_writer_el2.size bin_writer_el3.size v)
+        Size.bin_size_triple bin_writer_el1.size bin_writer_el2.size bin_writer_el3.size v)
   ; write =
       (fun buf ~pos v ->
-         Write.bin_write_triple
-           bin_writer_el1.write
-           bin_writer_el2.write
-           bin_writer_el3.write
-           buf
-           ~pos
-           v)
+        Write.bin_write_triple
+          bin_writer_el1.write
+          bin_writer_el2.write
+          bin_writer_el3.write
+          buf
+          ~pos
+          v)
   }
 ;;
 
 let bin_reader_triple bin_reader_el1 bin_reader_el2 bin_reader_el3 =
   { read =
       (fun buf ~pos_ref ->
-         Read.bin_read_triple
-           bin_reader_el1.read
-           bin_reader_el2.read
-           bin_reader_el3.read
-           buf
-           ~pos_ref)
+        Read.bin_read_triple
+          bin_reader_el1.read
+          bin_reader_el2.read
+          bin_reader_el3.read
+          buf
+          ~pos_ref)
   ; vtag_read = variant_wrong_type "triple"
   }
 ;;
@@ -348,14 +312,14 @@ let bin_writer_hashtbl bin_writer_el1 bin_writer_el2 =
   { size = (fun v -> Size.bin_size_hashtbl bin_writer_el1.size bin_writer_el2.size v)
   ; write =
       (fun buf ~pos v ->
-         Write.bin_write_hashtbl bin_writer_el1.write bin_writer_el2.write buf ~pos v)
+        Write.bin_write_hashtbl bin_writer_el1.write bin_writer_el2.write buf ~pos v)
   }
 ;;
 
 let bin_reader_hashtbl bin_reader_el1 bin_reader_el2 =
   { read =
       (fun buf ~pos_ref ->
-         Read.bin_read_hashtbl bin_reader_el1.read bin_reader_el2.read buf ~pos_ref)
+        Read.bin_read_hashtbl bin_reader_el1.read bin_reader_el2.read buf ~pos_ref)
   ; vtag_read = variant_wrong_type "hashtbl"
   }
 ;;
