@@ -19,6 +19,7 @@ module Write = Bin_prot.Write
 
 type 'a to_test =
   { writer : 'a Write.writer
+  ; writer_local : 'a Write.writer_local option
   ; reader : 'a Read.reader
   ; values : 'a list
   ; equal : 'a -> 'a -> bool
@@ -72,6 +73,7 @@ module Tests = struct
   (* max and min *)
   let unit =
     { writer = Write.bin_write_unit
+    ; writer_local = Some Write.bin_write_unit__local
     ; reader = Read.bin_read_unit
     ; values = [ () ]
     ; equal = Unit.equal
@@ -83,6 +85,7 @@ module Tests = struct
 
   let bool =
     { writer = Write.bin_write_bool
+    ; writer_local = Some Write.bin_write_bool__local
     ; reader = Read.bin_read_bool
     ; values = [ true; false ]
     ; equal = Bool.equal
@@ -94,6 +97,7 @@ module Tests = struct
 
   let char =
     { writer = Write.bin_write_char
+    ; writer_local = Some Write.bin_write_char__local
     ; reader = Read.bin_read_char
     ; values = [ '\x00'; 'A'; 'z'; ';'; '\xFF' ]
     ; equal = Char.equal
@@ -105,6 +109,7 @@ module Tests = struct
 
   let digest =
     { writer = Write.bin_write_md5
+    ; writer_local = Some Write.bin_write_md5__local
     ; reader = Read.bin_read_md5
     ; values = [ Md5_lib.of_hex_exn "0123456789abcdef0123456789ABCDEF" ]
     ; equal = (fun d1 d2 -> Md5_lib.compare d1 d2 = 0)
@@ -116,6 +121,7 @@ module Tests = struct
 
   let float =
     { writer = Write.bin_write_float
+    ; writer_local = Some Write.bin_write_float__local
     ; reader = Read.bin_read_float
     ; values =
         [ Float.epsilon_float
@@ -143,6 +149,7 @@ module Tests = struct
   (* min only *)
   let vec =
     { writer = Write.bin_write_vec
+    ; writer_local = Some Write.bin_write_vec__local
     ; reader = Read.bin_read_vec
     ; values = [ Vec.mk_float64 0; Vec.mk_float64 1 ]
     ; equal = Vec.equal
@@ -154,6 +161,7 @@ module Tests = struct
 
   let float32_vec =
     { writer = Write.bin_write_float32_vec
+    ; writer_local = Some Write.bin_write_float32_vec__local
     ; reader = Read.bin_read_float32_vec
     ; values = [ Vec.mk_float32 0; Vec.mk_float32 1 ]
     ; equal = Vec.equal
@@ -165,6 +173,7 @@ module Tests = struct
 
   let float64_vec =
     { writer = Write.bin_write_float64_vec
+    ; writer_local = Some Write.bin_write_float64_vec__local
     ; reader = Read.bin_read_float64_vec
     ; values = [ Vec.mk_float64 0; Vec.mk_float64 1 ]
     ; equal = Vec.equal
@@ -176,6 +185,7 @@ module Tests = struct
 
   let mat =
     { writer = Write.bin_write_mat
+    ; writer_local = Some Write.bin_write_mat__local
     ; reader = Read.bin_read_mat
     ; values = [ Mat.mk_float64 0 0; Mat.mk_float64 1 1 ]
     ; equal = Mat.equal
@@ -187,6 +197,7 @@ module Tests = struct
 
   let float32_mat =
     { writer = Write.bin_write_float32_mat
+    ; writer_local = Some Write.bin_write_float32_mat__local
     ; reader = Read.bin_read_float32_mat
     ; values = [ Mat.mk_float32 0 0; Mat.mk_float32 1 1 ]
     ; equal = Mat.equal
@@ -198,6 +209,7 @@ module Tests = struct
 
   let float64_mat =
     { writer = Write.bin_write_float64_mat
+    ; writer_local = Some Write.bin_write_float64_mat__local
     ; reader = Read.bin_read_float64_mat
     ; values = [ Mat.mk_float64 0 0; Mat.mk_float64 1 1 ]
     ; equal = Mat.equal
@@ -209,6 +221,7 @@ module Tests = struct
 
   let bigstring =
     { writer = Write.bin_write_bigstring
+    ; writer_local = Some Write.bin_write_bigstring__local
     ; reader = Read.bin_read_bigstring
     ; values = [ Bigstring.of_string ""; Bigstring.of_string "hello" ]
     ; equal =
@@ -221,6 +234,7 @@ module Tests = struct
 
   let floatarray =
     { writer = Write.bin_write_floatarray
+    ; writer_local = Some Write.bin_write_floatarray__local
     ; reader = Read.bin_read_floatarray
     ; values = [ Float_array.empty; Float_array.create ~len:1 0.0 ]
     ; equal = Float_array.equal Float.equal
@@ -232,6 +246,7 @@ module Tests = struct
 
   let ref =
     { writer = Write.bin_write_ref Write.bin_write_int32
+    ; writer_local = Some (Write.bin_write_ref__local Write.bin_write_int32__local)
     ; reader = Read.bin_read_ref Read.bin_read_int32
     ; values = [ ref 0l; ref 1l; ref (-1l); ref Int32.max_value; ref Int32.min_value ]
     ; equal = (fun v1 v2 -> !v1 = !v2)
@@ -243,6 +258,7 @@ module Tests = struct
 
   let lazy_t =
     { writer = Write.bin_write_lazy Write.bin_write_int32
+    ; writer_local = Some (Write.bin_write_lazy__local Write.bin_write_int32__local)
     ; reader = Read.bin_read_lazy Read.bin_read_int32
     ; values =
         [ lazy 0l; lazy 1l; lazy (-1l); lazy Int32.max_value; lazy Int32.min_value ]
@@ -255,6 +271,7 @@ module Tests = struct
 
   let option =
     { writer = Write.bin_write_option Write.bin_write_int32
+    ; writer_local = Some (Write.bin_write_option__local Write.bin_write_int32__local)
     ; reader = Read.bin_read_option Read.bin_read_int32
     ; values =
         [ None; Some 0l; Some 1l; Some (-1l); Some Int32.max_value; Some Int32.min_value ]
@@ -267,6 +284,11 @@ module Tests = struct
 
   let pair =
     { writer = Write.bin_write_pair Write.bin_write_int32 Write.bin_write_int32
+    ; writer_local =
+        Some
+          (Write.bin_write_pair__local
+             Write.bin_write_int32__local
+             Write.bin_write_int32__local)
     ; reader = Read.bin_read_pair Read.bin_read_int32 Read.bin_read_int32
     ; values =
         [ 0l, 0l
@@ -288,6 +310,12 @@ module Tests = struct
           Write.bin_write_int32
           Write.bin_write_int32
           Write.bin_write_int32
+    ; writer_local =
+        Some
+          (Write.bin_write_triple__local
+             Write.bin_write_int32__local
+             Write.bin_write_int32__local
+             Write.bin_write_int32__local)
     ; reader =
         Read.bin_read_triple Read.bin_read_int32 Read.bin_read_int32 Read.bin_read_int32
     ; values =
@@ -306,6 +334,7 @@ module Tests = struct
 
   let list =
     { writer = Write.bin_write_list Write.bin_write_int32
+    ; writer_local = Some (Write.bin_write_list__local Write.bin_write_int32__local)
     ; reader = Read.bin_read_list Read.bin_read_int32
     ; values =
         [ []
@@ -324,6 +353,7 @@ module Tests = struct
 
   let array =
     { writer = Write.bin_write_array Write.bin_write_int32
+    ; writer_local = Some (Write.bin_write_array__local Write.bin_write_int32__local)
     ; reader = Read.bin_read_array Read.bin_read_int32
     ; values =
         [ [||]
@@ -342,6 +372,7 @@ module Tests = struct
 
   let hashtbl =
     { writer = Write.bin_write_hashtbl Write.bin_write_int32 Write.bin_write_int32
+    ; writer_local = None
     ; reader = Read.bin_read_hashtbl Read.bin_read_int32 Read.bin_read_int32
     ; values =
         List.map
@@ -357,11 +388,11 @@ module Tests = struct
             hashtbl)
     ; equal =
         (fun t1 t2 ->
-           let to_list tbl =
-             Stdlib.Hashtbl.fold (fun k v acc -> (k, v) :: acc) tbl []
-             |> List.sort ~compare
-           in
-           to_list t1 = to_list t2)
+          let to_list tbl =
+            Stdlib.Hashtbl.fold (fun k v acc -> (k, v) :: acc) tbl []
+            |> List.sort ~compare
+          in
+          to_list t1 = to_list t2)
     ; sexp_of = [%sexp_of: (int32, int32) Sexplib.Std.Hashtbl.t]
     ; hi_bound = None
     ; lo_bound = Minimum.bin_size_hashtbl
@@ -373,7 +404,7 @@ module Tests = struct
       { x : Int32.t
       ; y : float
       }
-    [@@deriving bin_io, fields, sexp_of]
+    [@@deriving bin_io ~localize, fields ~iterators:fold, sexp_of]
 
     let maximum_bin_size =
       Fields.fold
@@ -393,6 +424,7 @@ module Tests = struct
   let record1 =
     let open R1 in
     { writer = bin_write_t
+    ; writer_local = Some bin_write_t__local
     ; reader = bin_read_t
     ; values = [ { x = 0l; y = 0.0 }; { x = Int32.max_value; y = Float.max_value } ]
     ; equal = ( = )
@@ -407,7 +439,7 @@ module Tests = struct
       { w : Int64.t
       ; x : Int32.t
       }
-    [@@deriving bin_io, fields, sexp_of]
+    [@@deriving bin_io ~localize, fields ~iterators:fold, sexp_of]
 
     let maximum_bin_size_of_inner =
       Fields_of_inner.fold
@@ -427,7 +459,7 @@ module Tests = struct
       { y : inner
       ; z : unit
       }
-    [@@deriving bin_io, fields, sexp_of]
+    [@@deriving bin_io ~localize, fields ~iterators:fold, sexp_of]
 
     let maximum_bin_size =
       Fields.fold
@@ -447,6 +479,7 @@ module Tests = struct
   let record2 =
     let open R2 in
     { writer = bin_write_t
+    ; writer_local = Some bin_write_t__local
     ; reader = bin_read_t
     ; values =
         [ { y = { w = 0L; x = 0l }; z = () }
@@ -466,7 +499,7 @@ module Tests = struct
           ; x : Int32.t
           }
       | Inner_other of unit
-    [@@deriving bin_io, sexp_of, variants]
+    [@@deriving bin_io ~localize, sexp_of, variants]
 
     let maximum_bin_size_of_inner =
       1
@@ -490,7 +523,7 @@ module Tests = struct
           { y : inner
           ; z : unit
           }
-    [@@deriving bin_io, sexp_of, variants]
+    [@@deriving bin_io ~localize, sexp_of, variants]
 
     let maximum_bin_size =
       1
@@ -514,6 +547,7 @@ module Tests = struct
   let inline_record =
     let open Inline_record in
     { writer = bin_write_t
+    ; writer_local = Some bin_write_t__local
     ; reader = bin_read_t
     ; values =
         [ Outer { y = Inner { w = 0L; x = 0l }; z = () }
@@ -535,7 +569,16 @@ let gen_tests t =
   let bin_protted_values =
     List.map t.values ~f:(fun v ->
       let len = t.writer buf ~pos:0 v in
-      Bigstring.To_string.sub buf ~pos:0 ~len)
+      let str = Bigstring.To_string.sub buf ~pos:0 ~len in
+      (match t.writer_local with
+       | None -> ()
+       | Some writer_local ->
+         let len = writer_local buf ~pos:0 v in
+         let str_local = Bigstring.To_string.sub buf ~pos:0 ~len in
+         if String.( <> ) str str_local
+         then
+           printf ", write_local output (%s) differs from write output (%s)" str_local str);
+      str)
   in
   let hex_size =
     List.fold bin_protted_values ~init:0 ~f:(fun acc s -> Int.max acc (String.length s))
@@ -546,24 +589,24 @@ let gen_tests t =
       bin_protted_values
       ~init:(Int.max_value, 0)
       ~f:(fun (min, max) v s ->
-        let len = String.length s in
-        printf !"%s -> %{Sexp}" (to_hex s hex_size) (t.sexp_of v);
-        Bigstring.From_string.blito ~src:s ~dst:buf ();
-        let pos_ref = ref 0 in
-        let v' = t.reader buf ~pos_ref in
-        let len' = !pos_ref in
-        let hi_bound = Option.value t.hi_bound ~default:Int.max_value in
-        if len < t.lo_bound || len > hi_bound
-        then printf ", bin_size outside of range %d..%d: %d" t.lo_bound hi_bound len;
-        if (not (t.equal v v')) || len <> len'
-        then
-          printf
-            !", read test failed: read %d byte%s as %{Sexp}"
-            len'
-            (if len' = 1 then "" else "s")
-            (t.sexp_of v');
-        Out_channel.output_char stdout '\n';
-        Int.min min len, Int.max max len)
+      let len = String.length s in
+      printf !"%s -> %{Sexp}" (to_hex s hex_size) (t.sexp_of v);
+      Bigstring.From_string.blito ~src:s ~dst:buf ();
+      let pos_ref = ref 0 in
+      let v' = t.reader buf ~pos_ref in
+      let len' = !pos_ref in
+      let hi_bound = Option.value t.hi_bound ~default:Int.max_value in
+      if len < t.lo_bound || len > hi_bound
+      then printf ", bin_size outside of range %d..%d: %d" t.lo_bound hi_bound len;
+      if (not (t.equal v v')) || len <> len'
+      then
+        printf
+          !", read test failed: read %d byte%s as %{Sexp}"
+          len'
+          (if len' = 1 then "" else "s")
+          (t.sexp_of v');
+      Out_channel.output_char stdout '\n';
+      Int.min min len, Int.max max len)
   in
   match t.hi_bound with
   | None ->
@@ -576,14 +619,12 @@ let gen_tests t =
 
 let%expect_test "Non-integer bin_prot size tests" =
   gen_tests Tests.unit;
-  [%expect {|
-    00 -> ()
-  |}];
+  [%expect {| 00 -> () |}];
   gen_tests Tests.bool;
   [%expect {|
     01 -> true
     00 -> false
-  |}];
+    |}];
   gen_tests Tests.char;
   [%expect
     {|
@@ -592,12 +633,10 @@ let%expect_test "Non-integer bin_prot size tests" =
     7a -> z
     3b -> ";"
     ff -> "\255"
-  |}];
+    |}];
   gen_tests Tests.digest;
   [%expect
-    {|
-    ef cd ab 89 67 45 23 01 ef cd ab 89 67 45 23 01 -> 0123456789abcdef0123456789abcdef
-  |}];
+    {| ef cd ab 89 67 45 23 01 ef cd ab 89 67 45 23 01 -> 0123456789abcdef0123456789abcdef |}];
   gen_tests Tests.float;
   [%expect
     {|
@@ -613,58 +652,58 @@ let%expect_test "Non-integer bin_prot size tests" =
     3f f0 00 00 00 00 00 00 -> 1
     3e 7a d7 f2 9a bc af 48 -> 1E-07
     00 00 00 00 00 00 00 00 -> 0
-  |}];
+    |}];
   gen_tests Tests.float_nan;
   Expect_test_patterns.require_match
     [%here]
     {|
     7f f{8,0} 00 00 00 00 00 01 -> NAN (glob)
-  |};
+    |};
   gen_tests Tests.vec;
   [%expect
     {|
     .. .. .. .. .. .. .. .. 00 -> ()
     3f f0 00 00 00 00 00 00 01 -> (1)
-  |}];
+    |}];
   gen_tests Tests.float32_vec;
   [%expect {|
     .. .. .. .. 00 -> ()
     3f 80 00 00 01 -> (1)
-  |}];
+    |}];
   gen_tests Tests.float64_vec;
   [%expect
     {|
     .. .. .. .. .. .. .. .. 00 -> ()
     3f f0 00 00 00 00 00 00 01 -> (1)
-  |}];
+    |}];
   gen_tests Tests.mat;
   [%expect
     {|
     .. .. .. .. .. .. .. .. 00 00 -> ()
     3f f0 00 00 00 00 00 00 01 01 -> ((1))
-  |}];
+    |}];
   gen_tests Tests.float32_mat;
   [%expect {|
     .. .. .. .. 00 00 -> ()
     3f 80 00 00 01 01 -> ((1))
-  |}];
+    |}];
   gen_tests Tests.float64_mat;
   [%expect
     {|
     .. .. .. .. .. .. .. .. 00 00 -> ()
     3f f0 00 00 00 00 00 00 01 01 -> ((1))
-  |}];
+    |}];
   gen_tests Tests.bigstring;
   [%expect {|
     .. .. .. .. .. 00 -> ""
     6f 6c 6c 65 68 05 -> hello
-  |}];
+    |}];
   gen_tests Tests.floatarray;
   [%expect
     {|
     .. .. .. .. .. .. .. .. 00 -> ()
     00 00 00 00 00 00 00 00 01 -> (0)
-  |}];
+    |}];
   gen_tests Tests.ref;
   [%expect
     {|
@@ -673,7 +712,7 @@ let%expect_test "Non-integer bin_prot size tests" =
     .. .. .. ff ff -> -1
     7f ff ff ff fd -> 2147483647
     80 00 00 00 fd -> -2147483648
-  |}];
+    |}];
   gen_tests Tests.lazy_t;
   [%expect
     {|
@@ -682,7 +721,7 @@ let%expect_test "Non-integer bin_prot size tests" =
     .. .. .. ff ff -> -1
     7f ff ff ff fd -> 2147483647
     80 00 00 00 fd -> -2147483648
-  |}];
+    |}];
   gen_tests Tests.option;
   [%expect
     {|
@@ -692,7 +731,7 @@ let%expect_test "Non-integer bin_prot size tests" =
     .. .. .. ff ff 01 -> (-1)
     7f ff ff ff fd 01 -> (2147483647)
     80 00 00 00 fd 01 -> (-2147483648)
-  |}];
+    |}];
   gen_tests Tests.pair;
   [%expect
     {|
@@ -701,7 +740,7 @@ let%expect_test "Non-integer bin_prot size tests" =
     .. .. .. .. .. .. ff ff ff ff -> (-1 -1)
     7f ff ff ff fd 7f ff ff ff fd -> (2147483647 2147483647)
     80 00 00 00 fd 80 00 00 00 fd -> (-2147483648 -2147483648)
-  |}];
+    |}];
   gen_tests Tests.triple;
   [%expect
     {|
@@ -710,7 +749,7 @@ let%expect_test "Non-integer bin_prot size tests" =
     .. .. .. .. .. .. .. .. .. ff ff ff ff ff ff -> (-1 -1 -1)
     7f ff ff ff fd 7f ff ff ff fd 7f ff ff ff fd -> (2147483647 2147483647 2147483647)
     80 00 00 00 fd 80 00 00 00 fd 80 00 00 00 fd -> (-2147483648 -2147483648 -2147483648)
-  |}];
+    |}];
   gen_tests Tests.list;
   [%expect
     {|
@@ -720,7 +759,7 @@ let%expect_test "Non-integer bin_prot size tests" =
     .. .. .. .. .. .. .. ff ff 01 02 -> (1 -1)
     .. .. .. .. 7f ff ff ff fd 00 02 -> (0 2147483647)
     80 00 00 00 fd 7f ff ff ff fd 02 -> (2147483647 -2147483648)
-  |}];
+    |}];
   gen_tests Tests.array;
   [%expect
     {|
@@ -730,24 +769,27 @@ let%expect_test "Non-integer bin_prot size tests" =
     .. .. .. .. .. .. .. ff ff 01 02 -> (1 -1)
     .. .. .. .. 7f ff ff ff fd 00 02 -> (0 2147483647)
     80 00 00 00 fd 7f ff ff ff fd 02 -> (2147483647 -2147483648)
-  |}];
+    |}];
   gen_tests Tests.record1;
   [%expect
     {|
     .. .. .. .. 00 00 00 00 00 00 00 00 00 -> ((x 0)(y 0))
-    7f f0 00 00 00 00 00 00 7f ff ff ff fd -> ((x 2147483647)(y INF)) |}];
+    7f f0 00 00 00 00 00 00 7f ff ff ff fd -> ((x 2147483647)(y INF))
+    |}];
   gen_tests Tests.record2;
   [%expect
     {|
     .. .. .. .. .. .. .. .. .. .. .. .. 00 00 00 -> ((y((w 0)(x 0)))(z()))
-    00 7f ff ff ff fd 7f ff ff ff ff ff ff ff fc -> ((y((w 9223372036854775807)(x 2147483647)))(z())) |}];
+    00 7f ff ff ff fd 7f ff ff ff ff ff ff ff fc -> ((y((w 9223372036854775807)(x 2147483647)))(z()))
+    |}];
   gen_tests Tests.inline_record;
   [%expect
     {|
     .. .. .. .. .. .. .. .. .. .. .. .. 00 00 00 00 01 -> (Outer(y(Inner(w 0)(x 0)))(z()))
     00 7f ff ff ff fd 7f ff ff ff ff ff ff ff fc 00 01 -> (Outer(y(Inner(w 9223372036854775807)(x 2147483647)))(z()))
     .. .. .. .. .. .. .. .. .. .. .. .. .. 00 00 01 01 -> (Outer(y(Inner_other()))(z()))
-    .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. 00 00 -> (Outer_other()) |}]
+    .. .. .. .. .. .. .. .. .. .. .. .. .. .. .. 00 00 -> (Outer_other())
+    |}]
 ;;
 
 (* Polymorphic hash is not the same when running in javascript.
@@ -761,5 +803,5 @@ let%expect_test ("Non-integer bin_prot size tests (no js)" [@tags "no-js"]) =
     .. .. .. .. .. .. .. .. .. .. 00 00 01 01 02 -> ((0 0)(1 1))
     .. .. 00 00 7f ff ff ff fd 7f ff ff ff fd 02 -> ((0 0)(2147483647 2147483647))
     80 00 00 00 fd 80 00 00 00 fd ff ff ff ff 02 -> ((-2147483648 -2147483648)(-1 -1))
-  |}]
+    |}]
 ;;
